@@ -2,19 +2,10 @@
 // All rights reserved.
 //
 // This file is part of CGAL (www.cgal.org).
-// You can redistribute it and/or modify it under the terms of the GNU
-// General Public License as published by the Free Software Foundation,
-// either version 3 of the License, or (at your option) any later version.
 //
-// Licensees holding a valid commercial license may use this file in
-// accordance with the commercial license agreement provided with the software.
-//
-// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-//
-// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.3/Polyhedron/demo/Polyhedron/include/Point_set_3.h $
-// $Id: Point_set_3.h 2f81a21 2019-07-28T18:38:59+02:00 Sébastien Loriot
-// SPDX-License-Identifier: GPL-3.0+
+// $URL: https://github.com/CGAL/cgal/blob/v5.5.2/Polyhedron/demo/Polyhedron/include/Point_set_3.h $
+// $Id: Point_set_3.h 5f18034 2022-01-06T09:16:55+01:00 Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
 // Author(s)     : Laurent Saboret, Nader Salman, Gael Guennebaud, Simon Giraudot
@@ -53,7 +44,7 @@ template <class Gt>
 class Point_set_3 : public CGAL::Point_set_3<typename Gt::Point_3,
                                              typename Gt::Vector_3>
 {
-  
+
 public:
 
   typedef Gt  Geom_traits; ///< Geometric traits class.
@@ -69,14 +60,14 @@ public:
   typedef typename Base::iterator iterator;
   typedef typename Base::const_iterator const_iterator;
   typedef typename Base::Index Index;
-  
+
   // Classic CGAL geometric types
 
   typedef typename Base::template Property_map<double> Double_map;
   typedef typename Base::template Property_map<unsigned char> Byte_map;
-  
+
 private:
-  
+
   // Indicate if m_barycenter, m_bounding_box, m_bounding_sphere and
   // m_diameter_standard_deviation below are valid.
   mutable bool m_bounding_box_is_valid;
@@ -94,10 +85,10 @@ private:
   Double_map m_fred;
   Double_map m_fgreen;
   Double_map m_fblue;
-  
+
   mutable CGAL::Iterator_range<const_iterator> m_const_range;
   CGAL::Iterator_range<iterator> m_range;
-  
+
   // Assignment operator not implemented and declared private to make
   // sure nobody uses the default one without knowing it
   Point_set_3& operator= (const Point_set_3&)
@@ -105,9 +96,9 @@ private:
     return *this;
   }
 
-  
+
 public:
-  
+
   Point_set_3 ()
     : m_const_range (begin(), end())
     , m_range (begin(), end())
@@ -116,7 +107,7 @@ public:
     m_radii_are_uptodate = false;
   }
 
-  // copy constructor 
+  // copy constructor
   Point_set_3 (const Point_set_3& p)
     : Base (p)
     , m_const_range (begin(), end())
@@ -140,7 +131,7 @@ public:
   {
     this->cancel_removals();
   }
-  
+
   bool add_radius()
   {
     bool out = false;
@@ -153,7 +144,7 @@ public:
   bool check_colors()
   {
     bool found = false;
-    
+
     boost::tie (m_red, found) = this->template property_map<unsigned char>("red");
     if (!found)
       {
@@ -214,10 +205,10 @@ public:
   bool get_las_colors()
   {
     bool found = false;
-    
+
     typedef typename Base::template Property_map<unsigned short> Ushort_map;
     Ushort_map red, green, blue;
-    
+
     boost::tie (red, found) = this->template property_map<unsigned short>("R");
     if (!found)
       return false;
@@ -252,7 +243,7 @@ public:
     this->remove_property_map(red);
     this->remove_property_map(green);
     this->remove_property_map(blue);
-    
+
     return true;
   }
 
@@ -270,14 +261,14 @@ public:
   {
     if (has_colors())
       return false;
-    
+
     m_red = this->template add_property_map<unsigned char>("red", 0).first;
     m_green = this->template add_property_map<unsigned char>("green", 0).first;
     m_blue = this->template add_property_map<unsigned char>("blue", 0).first;
 
     return true;
   }
-  
+
   void remove_colors()
   {
     if (m_blue != Byte_map())
@@ -300,7 +291,7 @@ public:
   { return (m_green == Byte_map()) ? m_fgreen[index]  : double(m_green[index]) / 255.; }
   double blue (const Index& index) const
   { return (m_blue == Byte_map()) ? m_fblue[index]  : double(m_blue[index]) / 255.; }
-  
+
   void set_color (const Index& index, unsigned char r = 0, unsigned char g = 0, unsigned char b = 0)
   {
     m_red[index] = r;
@@ -322,8 +313,8 @@ public:
     m_green[index] = color[1];
     m_blue[index] = color[2];
   }
-    
-  
+
+
   iterator first_selected() { return this->m_indices.end() - this->m_nb_removed; }
   const_iterator first_selected() const { return this->m_indices.end() - this->m_nb_removed; }
   void set_first_selected(iterator it)
@@ -357,7 +348,7 @@ public:
   {
     return this->is_removed (it);
   }
-  
+
   // Test if point is selected
   bool is_selected(const Index& idx) const
   {
@@ -405,7 +396,7 @@ public:
 
     while (sel != first - 1 && unsel != first)
       std::swap (*(sel --), *(unsel ++));
-    
+
     this->m_nb_removed = size() - this->m_nb_removed;
   }
 
@@ -501,10 +492,10 @@ public:
   {
     m_bounding_box_is_valid = false;
   }
-  
+
   bool are_radii_uptodate() const { return m_radii_are_uptodate; }
   void set_radii_uptodate(bool /*on*/) { m_radii_are_uptodate = false; }
-  
+
   CGAL::Named_function_parameters
   <Kernel,
    CGAL::internal_np::geom_traits_t,
@@ -539,7 +530,7 @@ private:
     for (const_iterator it = begin(); it != end(); it++)
     {
       const Point& p = this->point(*it);
-      
+
       // update bbox
       xmin = (std::min)(p.x(),xmin);
       ymin = (std::min)(p.y(),ymin);
@@ -581,38 +572,18 @@ private:
 
     m_bounding_box_is_valid = true;
   }
-  
+
 }; // end of class Point_set_3
 
 namespace CGAL
 {
-namespace Point_set_processing_3
-{
-  template<typename Kernel>
-  class GetFT< ::Point_set_3<Kernel> >
-  {
-  public:
-    typedef typename Kernel::FT type;
-  };
-  
-  namespace parameters
-  {
-    template <typename Kernel>
-    Named_function_parameters
-    <Kernel,
-     internal_np::geom_traits_t,
-     Named_function_parameters
-     <typename ::Point_set_3<Kernel>::template Property_map<typename Kernel::Vector_3>,
-      internal_np::normal_t,
-      Named_function_parameters
-      <typename ::Point_set_3<Kernel>::template Property_map<typename Kernel::Point_3>,
-       internal_np::point_t> > >
-    inline all_default(const ::Point_set_3<Kernel>& ps)
-    {
-      return ps.parameters();
-    }
-  }
-}
+
+// specialization for default named parameters
+template <typename Gt, typename NamedParameters, typename NP_TAG>
+struct Point_set_processing_3_np_helper<::Point_set_3<Gt>, NamedParameters, NP_TAG>
+    : public Point_set_processing_3_np_helper<typename ::Point_set_3<Gt>::Base, NamedParameters, NP_TAG>
+{};
+
 }
 
 #endif // POINT_SET_3_H
