@@ -3,8 +3,8 @@
 //
 // This file is part of CGAL (www.cgal.org).
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.5/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Face_graph_output_builder.h $
-// $Id: Face_graph_output_builder.h a8621f3 2022-05-10T15:05:10+02:00 Sébastien Loriot
+// $URL: https://github.com/CGAL/cgal/blob/v5.5.2/Polygon_mesh_processing/include/CGAL/Polygon_mesh_processing/internal/Corefinement/Face_graph_output_builder.h $
+// $Id: Face_graph_output_builder.h 119cea4 2023-01-27T20:52:27+01:00 Laurent Rineau
 // SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
 //
 //
@@ -1037,12 +1037,69 @@ public:
           std::size_t patch_id_q1=tm2_patch_ids[ get(fids2, face(opposite(h2,tm2),tm2)) ];
           std::size_t patch_id_q2=tm2_patch_ids[ get(fids2, face(h2,tm2)) ];
 
+          // info on whether the patches were already classified
+          std::bitset<4> patch_status_was_not_already_set;
+          std::bitset<4> previous_bitvalue;
+          // info for tm1
+          patch_status_was_not_already_set[0] = patch_status_not_set_tm1.test(patch_id_p1);
+          patch_status_was_not_already_set[1] = patch_status_not_set_tm1.test(patch_id_p2);
+          previous_bitvalue[0] = is_patch_inside_tm2.test(patch_id_p1);
+          previous_bitvalue[1] = is_patch_inside_tm2.test(patch_id_p2);
+          // info for tm2
+          patch_status_was_not_already_set[2] = patch_status_not_set_tm2.test(patch_id_q1);
+          patch_status_was_not_already_set[3] = patch_status_not_set_tm2.test(patch_id_q2);
+          previous_bitvalue[2] = is_patch_inside_tm1.test(patch_id_q1);
+          previous_bitvalue[3] = is_patch_inside_tm1.test(patch_id_q2);
+
+#ifndef CGAL_NDEBUG
+          if (is_tm1_closed && is_tm2_closed)
+          {
+            if (!patch_status_was_not_already_set[0] &&
+                !patch_status_was_not_already_set[1] &&
+                !patch_status_was_not_already_set[2] &&
+                !patch_status_was_not_already_set[3])
+              continue; // all patches were already classified, no need to redo it
+          }
+#endif
+
+          // check incompatibility of patch classifications
+          auto inconsistent_classification = [&]()
+          {
+            if (!used_to_clip_a_surface && !used_to_classify_patches && (!is_tm1_closed || !is_tm2_closed))
+            {
+              //make sure there is no ambiguity in tm1
+              if( (!patch_status_was_not_already_set[0] && previous_bitvalue[0]!=is_patch_inside_tm2.test(patch_id_p1) ) ||
+                  (!patch_status_was_not_already_set[1] && previous_bitvalue[1]!=is_patch_inside_tm2.test(patch_id_p2) ) )
+              {
+                impossible_operation.set();
+                return true;
+              }
+              //make sure there is no ambiguity in tm2
+              if( (!patch_status_was_not_already_set[2] && previous_bitvalue[2]!=is_patch_inside_tm1.test(patch_id_q1) ) ||
+                  (!patch_status_was_not_already_set[3] && previous_bitvalue[3]!=is_patch_inside_tm1.test(patch_id_q2) ) )
+              {
+                impossible_operation.set();
+                return true;
+              }
+            }
+            return false;
+          };
+
           //indicates that patch status will be updated
           patch_status_not_set_tm1.reset(patch_id_p1);
           patch_status_not_set_tm1.reset(patch_id_p2);
           patch_status_not_set_tm2.reset(patch_id_q1);
           patch_status_not_set_tm2.reset(patch_id_q2);
 
+          // restore initial state, needed when checking in `inconsistent_classification()`
+          if (!is_tm1_closed  || !is_tm2_closed)
+          {
+            is_patch_inside_tm2.reset(patch_id_p1);
+            is_patch_inside_tm2.reset(patch_id_p2);
+            is_patch_inside_tm1.reset(patch_id_q1);
+            is_patch_inside_tm1.reset(patch_id_q2);
+          }
+
 #ifdef CGAL_COREFINEMENT_POLYHEDRA_DEBUG
           #warning: Factorize the orientation predicates.
 #endif //CGAL_COREFINEMENT_POLYHEDRA_DEBUG
@@ -1082,6 +1139,7 @@ public:
 
             if ( q2_is_between_p1p2 ) is_patch_inside_tm1.set(patch_id_q2); //case 1
             else is_patch_inside_tm2.set(patch_id_p2); //case 2
+            if (inconsistent_classification()) return;
             continue;
           }
           else{
@@ -1112,6 +1170,7 @@ public:
                 is_patch_inside_tm1.set(patch_id_q1);
                 is_patch_inside_tm2.set(patch_id_p2);
               } //else case 4
+              if (inconsistent_classification()) return;
               continue;
             }
             else
@@ -1142,6 +1201,7 @@ public:
                   is_patch_inside_tm1.set(patch_id_q2);
                   is_patch_inside_tm2.set(patch_id_p1);
                 } // else case 6
+                if (inconsistent_classification()) return;
                 continue;
               }
               else{
@@ -1170,6 +1230,7 @@ public:
 
                   if ( q1_is_between_p1p2 ) is_patch_inside_tm1.set(patch_id_q1);  //case 7
                   else is_patch_inside_tm2.set(patch_id_p1); //case 8
+                  if (inconsistent_classification()) return;
                   continue;
                 }
               }
@@ -1331,6 +1392,14 @@ public:
               }
             }
           }
+          if (inconsistent_classification()) return;
+          if (!used_to_clip_a_surface && !used_to_classify_patches)
+          {
+            CGAL_assertion( patch_status_was_not_already_set[0] || previous_bitvalue[0]==is_patch_inside_tm2[patch_id_p1] );
+            CGAL_assertion( patch_status_was_not_already_set[1] || previous_bitvalue[1]==is_patch_inside_tm2[patch_id_p2] );
+            CGAL_assertion( patch_status_was_not_already_set[2] || previous_bitvalue[2]==is_patch_inside_tm1[patch_id_q1] );
+            CGAL_assertion( patch_status_was_not_already_set[3] || previous_bitvalue[3]==is_patch_inside_tm1[patch_id_q2] );
+          }
         }
     }
 
