@@ -7,8 +7,8 @@
 //
 // This file is part of CGAL (www.cgal.org)
 //
-// $URL: https://github.com/CGAL/cgal/blob/v5.5/Installation/include/CGAL/version.h $
-// $Id: version.h 10f5fa4 2022-06-29T14:52:29+02:00 Laurent Rineau
+// $URL: https://github.com/CGAL/cgal/blob/v5.5.2/Installation/include/CGAL/version.h $
+// $Id: version.h 755bfe4 2022-10-12T15:30:32+02:00 Laurent Rineau
 // SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
 //
 // Author(s)     : -
@@ -17,12 +17,12 @@
 #define CGAL_VERSION_H
 
 #ifndef SWIG
-#define CGAL_VERSION 5.5
-#define CGAL_GIT_HASH 80c30e4c79f1bb34673cdcfa22d8498d7c09967c
+#define CGAL_VERSION 5.5.2
+#define CGAL_GIT_HASH 8a3184a1a82c3e7d737656bfa4950471c369a4b9
 #endif
-#define CGAL_VERSION_NR 1050501000
+#define CGAL_VERSION_NR 1050521000
 #define CGAL_SVN_REVISION 99999
-#define CGAL_RELEASE_DATE 20220713
+#define CGAL_RELEASE_DATE 20230224
 
 #include <CGAL/version_macros.h>
 
